--*******************************************************************************************************************************
--
-- Description: This module throttles data flow based on an external enable or a requested sample count.
--					
-- Rev History: 3-31-2014:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

use work.model_specific_package.all;
use work.transceiver_package.all;
use work.tb_package.all;
use work.dma_package.all;

entity pci_express is
port (		
	-- PCI Express reset, clock and data signals(which are not used for the simulation).
	sys_rst_n		: in std_logic;
    pci_sys_clk		: in std_logic;
    pci_exp_rxp		: in std_logic_vector (PCI_LANES-1 downto 0);
    pci_exp_rxn		: in std_logic_vector (PCI_LANES-1 downto 0);
    pci_exp_txp		: out std_logic_vector (PCI_LANES-1 downto 0);
    pci_exp_txn		: out std_logic_vector (PCI_LANES-1 downto 0);
    
    -- Generated clock and reset.
    user_clk		: out std_logic;
    user_reset		: out std_logic;
    
    -- Slave (slv) interface connections.
	slv_addr 		: out std_logic_vector(63 downto 0);
	slv_dataout 	: out std_logic_vector(AXI_WIDTH-1 downto 0);
	slv_bytevalid 	: out std_logic_vector((AXI_WIDTH/8)-1 downto 0);
	bar0_rddata 	: in std_logic_vector(AXI_WIDTH-1 downto 0);
	bar0_write 		: out std_logic;
	bar0_read 		: out std_logic;
	bar2_rddata 	: in std_logic_vector(AXI_WIDTH-1 downto 0);
	bar2_write 		: out std_logic;
	bar2_read 		: out std_logic;

	-- Direct memory access (dma) interface connections to the application logic.
	dma_wr 			: out std_logic;
	dma_rd 			: out std_logic;
	dma_rdaddr 		: out std_logic_vector(15 downto 0);
	dma_wrchannel 	: out std_logic_vector(15 downto 0);
	dma_rdchannel 	: out std_logic_vector(15 downto 0);
	dma_regin		: in DMA_TYPE_128;
	dma_param		: in DMA_TYPE_24;
	dma_control		: in DMA_TYPE_6; --std_logic_vector(5 downto 0);
	dma_status		: out DMA_TYPE_4;
	dma_fifocnt		: in DMA_TYPE_13;
	dma_rddata		: in DMA_TYPE_AXI;
	dma_wrdata		: out std_logic_vector(AXI_WIDTH-1 downto 0);
	
    -- Interrupt controller signals.
	int_request		: in std_logic;
	int_timeout		: in std_logic_vector(27 downto 0);
	int_response	: out std_logic_vector(27 downto 0)
	);                                   
end pci_express;   

architecture structural of pci_express is
	   
    -- Clock and reset signals
    signal user_clk_r	: std_logic; -- Clock created to similate the clock created by the PCI Express core. (125 MHz or 250 MHz)
    signal user_reset_r	: std_logic; -- Active high reset signal created to similate the reset created by the PCI Express core.
    signal pci_rst		: std_logic;
	
	-- Control Signals                               
    signal dma_rd_ch	: std_logic_vector(RX_CHANNEL_COUNT downto 0); 
    signal dma_mem_wr	: std_logic_vector(RX_CHANNEL_COUNT+TX_CHANNEL_COUNT downto RX_CHANNEL_COUNT+1); 
    signal target_read	: std_logic_vector(1 downto 0);
    signal tx_buff_full	: std_logic_vector(RX_CHANNEL_COUNT+TX_CHANNEL_COUNT downto RX_CHANNEL_COUNT+1); -- Signal used to make sure the transmit buffer is loaded with data before
  	signal tx_wrdata	: DMA_TYPE_AXI;

begin                                                          
       
	-------------------------------------------
	-- clock generator
	-------------------------------------------
             
	USER_CLK_GEN : process
	begin	
        user_clk_r <= '1';
        wait for USER_CLK_PD/2.0;                         
        loop                                          
            user_clk_r <= '0', '1' after USER_CLK_PD/2.0;
            wait for USER_CLK_PD;                                                            
        end loop;         
	end process;
	
    USER_RST_GEN : process
	begin	
        user_reset_r <= '1';  
        wait for 750 ns;                         
        user_reset_r <= '0';    
        wait;                                                    
	end process;          
 
	user_clk	<= user_clk_r; 
	user_reset 	<= user_reset_r;
	
		
	RX_CH : if RX_CHANNEL_COUNT > 0 generate
		RX : for i in 1 to zero_to_one(RX_CHANNEL_COUNT) generate
			STATE_MACHINE : entity work.rx_dma_emulator 
			   	port map(		                                                                                 
	    			user_clk		=> user_clk_r,
	    			user_reset		=> user_reset_r,
					dma_rd_ch 		=> dma_rd_ch(i), 	
					dma_rdchannel 	=> dma_rdchannel(i), 
					dma_status		=> dma_status(i)(3),	
					dma_control		=> dma_control(i),       
					dma_rddata		=> dma_rddata(i)
				);
		end generate;
	end generate;
	
	TX_CH : if TX_CHANNEL_COUNT > 0 generate
		TX : for i in RX_CHANNEL_COUNT+1 to TX_CHANNEL_COUNT+RX_CHANNEL_COUNT generate
			STATE_MACHINE : entity work.tx_dma_emulator 
			   	port map(		                                                                                 
    				user_clk		=> user_clk_r,
    				user_reset		=> user_reset_r,
					tx_buff_full	=> tx_buff_full(i),
					dma_mem_wr 		=> dma_mem_wr(i), 	
					dma_wrchannel 	=> dma_wrchannel(i), 
					dma_status		=> dma_status(i)(3),	
					dma_control		=> dma_control(i),       
					dma_wrdata		=> tx_wrdata(i) -- ** Need to look into this **
				);
		end generate;
		
		-- ** Needs more looking into. **
		dma_wrdata 		<= tx_wrdata(3);
	end generate;
	
	dma_rd_ch(0) 	<= target_read(0) or target_read(1);
	bar0_read		<= target_read(0); 		
	bar2_read 		<= target_read(1);		
	dma_rd 			<= '1' when to_integer(unsigned(dma_rd_ch)) /= 0 else '0';
 	dma_wr 			<= '1' when to_integer(unsigned(dma_mem_wr)) /= 0 else '0';
 	
 	pci_rst			<= user_reset_r when TX_CHANNEL_COUNT = 0 else
 					   '1' when to_integer(unsigned(tx_buff_full)) = 0 else '0';
 					   
 					   
	BAR_TRANSACTION : entity work.pci_target_emulator 
		port map(		
    		user_clk		=> user_clk_r,
    		user_reset		=> pci_rst,
			slv_addr 		=> slv_addr, 		    
			slv_dataout 	=> slv_dataout, 	
			slv_bytevalid 	=> slv_bytevalid, 	
			dma_rdaddr 		=> dma_rdaddr,
			dma_rdchannel 	=> dma_rdchannel(0), 
			bar0_rddata 	=> bar0_rddata, 	
			bar0_write 		=> bar0_write, 		
			bar0_read 		=> target_read(0), 		
			bar2_rddata 	=> bar2_rddata, 	
			bar2_write 		=> bar2_write, 		
			bar2_read 		=> target_read(1) 		
		);
		
                     
end structural;

