--*******************************************************************************************************************************
--
-- Description: This module controls target dma reads and writes, emulating functions controlled by the PLDA EZDMA2 core in the SigFPGA sample project.
--					
-- Rev History: 3-31-2014:	Initial release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

use work.model_specific_package.all;
use work.transceiver_package.all;
use work.tb_package.all;

entity pci_target_emulator is
port (		
    -- Clock and reset.
    user_clk		: in std_logic;
    user_reset		: in std_logic;
    
    -- Slave (slv) interface connections.
	slv_addr 		: out std_logic_vector(63 downto 0);
	slv_dataout 	: out std_logic_vector(AXI_WIDTH-1 downto 0);
	slv_bytevalid 	: out std_logic_vector((AXI_WIDTH/8)-1 downto 0);
	dma_rdaddr 		: out std_logic_vector(15 downto 0);
	dma_rdchannel 	: out std_logic;
	bar0_rddata 	: in std_logic_vector(AXI_WIDTH-1 downto 0);
	bar0_write 		: out std_logic;
	bar0_read 		: out std_logic;
	bar2_rddata 	: in std_logic_vector(AXI_WIDTH-1 downto 0);
	bar2_write 		: out std_logic;
	bar2_read 		: out std_logic
	);
end pci_target_emulator;   

architecture structural of pci_target_emulator is
	signal dma_descriptor 	: std_logic_vector(AXI_WIDTH-1 downto 0);	-- Descriptor assigned to the active DMA page.
	signal dma_ch_select  	: std_logic_vector(15 downto 0);			-- Select the active channel.
    signal ONE				: std_logic_vector(63 downto 0);
    
begin
	ONE  <=(others=>'1');
	
	dma_descriptor(AXI_WIDTH-1 downto 32)	<= (others => '0'); 
	dma_descriptor(31 downto 30)			<= "11"; -- Set descriptor to one page and to stop on a count.
	dma_descriptor(29 downto 16)			<= std_logic_vector(to_unsigned(BURST_SIZE,14)); -- Assign burst size set in the tb_package.
	dma_descriptor(15 downto 0)				<= std_logic_vector(to_unsigned(BURST_COUNT,16)); -- Assign burst count set in the tb_package.
	
	BAR_TRANSACTION : process
	begin  
		slv_addr 		<= (others => '0');
		slv_dataout 	<= (others => '0');
		slv_bytevalid 	<= (others => '0');
		dma_rdaddr 		<= (others => '0');
		dma_rdchannel 	<= '0';
		bar0_write 		<= '0';
		bar0_read 		<= '0';        
		bar2_write 		<= '0';	            
		bar2_read 		<= '0';
		dma_ch_select	<= (others => '0');
		
		wait until user_reset = '0' and rising_edge(user_clk);   
		
		---------------------------------------------
		--	Bar 0 Write
		---------------------------------------------
		-- Select the DMA channel and write the descriptor, burst size, burst count, and number of pages. Loop for all active channels. 
		for i in 1 to TX_CHANNEL_COUNT+RX_CHANNEL_COUNT loop
			
			dma_ch_select		<= (others => '0');
			dma_ch_select(i+8)	<= '1'; 
 		
			wait for USER_CLK_PD; -- Wait one clock period between write commands. 
			-- Select DMA channel
			slv_addr 		<= x"0000000000001FF8";
			slv_dataout((AXI_WIDTH-64)+63 downto (AXI_WIDTH-64)+16) <= (others => '0');
			slv_dataout((AXI_WIDTH-64)-1 downto 0) 					<= (others => '0');			
			slv_dataout((AXI_WIDTH-64)+15 downto (AXI_WIDTH-64)) 	<= dma_ch_select;
			slv_bytevalid 	<= ONE((AXI_WIDTH/8)-1 downto 0);
			bar0_write 		<= '1';
			
			wait for USER_CLK_PD*2;  -- Wait for 2 clocks to allow a write to happen.			
			
			-- Write the burst size, burst count, last page, and interrupt enable.
			slv_addr 		<= x"0000000000002000";
			slv_dataout 	<= dma_descriptor;
			slv_bytevalid 	<= ONE((AXI_WIDTH/8)-1 downto 0);
			bar0_write 		<= '1';
			
			wait for USER_CLK_PD*2;  -- Wait to allow a write to happen.
		end loop;
		

		-- Deselect all DMA channels.             
		slv_addr 		<= x"0000000000001FF8";
		slv_dataout 	<= (others => '0');
		slv_bytevalid 	<= ONE((AXI_WIDTH/8)-1 downto 0);
		bar0_write 		<= '1';
		wait for USER_CLK_PD*2;   
		
		-- End Bar 0 write 
		bar0_write 		<= '0';
		
		
		---------------------------------------------
		--	Bar 2 Write
		---------------------------------------------
		-- Set start on software trigger, stop on count, and set sample count to 2048 for all active channels.
		for i in 1 to TX_CHANNEL_COUNT+RX_CHANNEL_COUNT loop
			slv_addr 		<= x"000000000000" & std_logic_vector(to_unsigned(i,4)) & x"020";
			slv_dataout 	<= x"0000000000000000000008000000000D";
			slv_bytevalid 	<= ONE((AXI_WIDTH/8)-1 downto 0);
			bar2_write 		<= '1';	
			
			wait for USER_CLK_PD*2;    
		end loop;
		     
		-- Activate the synchronizer for all active channels.
		for i in 1 to TX_CHANNEL_COUNT+RX_CHANNEL_COUNT loop
			slv_addr 		<= x"000000000000" & std_logic_vector(to_unsigned(i,4)) & x"000";
			slv_dataout 	<= x"00000000000000000000000000000001";
			slv_bytevalid 	<= ONE((AXI_WIDTH/8)-1 downto 0);  
			bar2_write 		<= '1';	

			wait for USER_CLK_PD*2;
		end loop;
		
		-- Turn off writes for bar 2.		
		slv_addr 		<= (others => '0');
		slv_dataout 	<= (others => '0');
		slv_bytevalid 	<= (others => '0');
		bar2_write 		<= '0';	
		
		-- Read sychronizer status for each active channel.
		for i in 1 to TX_CHANNEL_COUNT+RX_CHANNEL_COUNT loop
			dma_rdaddr 		<= std_logic_vector(to_unsigned(i,4)) & x"000";
			dma_rdchannel	<= '1';
			bar2_read 		<= '1';	
			                                       
			-- Wait 4 clock cycles after a read to make sure intended data is present. This will work for the longest latency possible from PLDA. 
			wait for USER_CLK_PD*4;
			
			-- Wait for channel sychronizer status bit 1 to go high and rising edge of clock.
			wait until bar2_rddata(1) = '1' and rising_edge(user_clk); 
			
		end loop; 
				
		dma_rdaddr 		<= (others => '0');
		dma_rdchannel	<= '0';
		bar2_read 		<= '0';	
		
		-- Fire the software trigger.
		slv_addr 		<= x"0000000000000010";
		slv_dataout 	<= x"00000000000000000000000000000000";
		slv_bytevalid 	<= ONE((AXI_WIDTH/8)-1 downto 0);
		bar2_write 		<= '1';	

		wait for USER_CLK_PD*2;    
		
		-- Turn off writes to Bar2
		slv_addr 		<= (others => '0');
		slv_dataout 	<= (others => '0');
		slv_bytevalid 	<= (others => '0');
		bar2_write 		<= '0';	
		
		-- Read Bar2 synchronizer status register for channel #1
		dma_rdaddr 		<= x"1000";
		bar2_read 		<= '1';	
		dma_rdchannel	<= '1';
		
		wait for USER_CLK_PD*4;
		wait until bar2_rddata(0) = '0' and rising_edge(user_clk); 
		
		-- Read Bar2 synchronizer status register for channel #2
		dma_rdaddr 		<= x"2000";
		bar2_read 		<= '1';	
		dma_rdchannel	<= '1';
		
		wait for USER_CLK_PD*4;
		wait until bar2_rddata(0) = '0' and rising_edge(user_clk); 

		-- Turn off reads to Bar2 and set simulation done bit high		
		dma_rdaddr 		<= (others => '0');
		dma_rdchannel	<= '0';
		bar2_read 		<= '0';	
		wait;		    
	end process;                                 
	
end structural;

